﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

namespace AT{
    namespace UBF{
        namespace Arbiters{

            /**  Selects single, highest-priority Action (winner takes all).
             *
             *  Warning:  If multiple actions have the same priority, this has undefined behavior!
             *  It actually returns the first Action with the highest priority, but Action order cannot be guaranteed.
             */
            public class Priority : AT.UBF.Arbiter {

                public override AT.UBF.Action selectAction(List<AT.UBF.Action> candidateActions){
                    if(candidateActions.Count < 1){
                        Debug.Log("Cannot select action from empty Actions list.");
                        return null;
                    }

                    AT.UBF.Action currAction = candidateActions[0];
                    float currMax = currAction.priority;

                    for(int i=1; i<candidateActions.Count; i++){
                        AT.UBF.Action next = candidateActions[i];
                        if(next.priority > currMax){
                            currAction = next;
                            currMax = next.priority;
                        }
                    }

                    return currAction;
                }
            }   
        }
    }
}

