using UnityEngine;
using System.Collections;
using System.Collections.Generic;

namespace AT{
    namespace UBF{
        namespace Arbiters{

            /** Children are considered in order, with the first child to vote being chosen. Winner takes all.
             *
             * SubsumptionWTA is essentially a Priority Arbiter where the order of the child Behaviors determine 
             * their priority. It's a matter of preference which you want to use. 
             */
            public class SubsumptionWTA : AT.UBF.Arbiter {

                public override AT.UBF.Action selectAction(List<AT.UBF.Action> candidateActions){
                    if(candidateActions.Count < 1){
                        Debug.Log("Cannot select action from empty Actions list.");
                        return null;
                    }

                    AT.UBF.Action winner;

                    for(int i=0; i<candidateActions.Count; i++){
                        winner = candidateActions[i];
                        if(!winner.IsNull()){
                            return winner;
                        }
                    }

                    return new AT.UBF.Action(); // return null/Abstain vote because none of our children voted
                }
            }   
        }
    }
}
