﻿using UnityEngine;
using System;
using System.Collections;
using System.Collections.Generic;

namespace AT{
    namespace UBF{
        namespace Behaviors{
            /** Tries to avoid crashing into walls by making the shortest turn.
             *
             *  FIXME: can get stuck in corners.
             */
            class AvoidWalls : AT.UBF.Behavior{
                public float cushionInSecondsToCrash = 1.0f; // seconds at current velocity to care about hitting a wall
                public float priorityOnPotentionalWallCrash = 1.0f;

                protected override AT.UBF.Action genActionInternal(AT.UBF.State state) {
                    AT.UBF.Action action = new AT.UBF.Action();

                    Vector2 velMagnitude = state.self.getVelocity() * cushionInSecondsToCrash;

                    Vector2 velocityLineEnd1 = state.bodyWidth.first + velMagnitude;
                    Vector2 velocityLineEnd2 = state.bodyWidth.second + velMagnitude;
                    Tuple<Vector2, Vector2> velocityLine1 = new Tuple<Vector2, Vector2>(state.bodyWidth.first, velocityLineEnd1);
                    Tuple<Vector2, Vector2> velocityLine2 = new Tuple<Vector2, Vector2>(state.bodyWidth.second, velocityLineEnd2);

                    foreach (Tuple<Vector2, Vector2> wall in state.walls) {
                        float? angle = LineIntersection.calculate(velocityLine1, wall);

                        if (!angle.HasValue) {
                            angle = LineIntersection.calculate(velocityLine2, wall);
                        }
                        if (angle.HasValue) {
                            action.SetPriority(priorityOnPotentionalWallCrash, voteSourceID);
                            action.SetTurnTracksRight(calculateTurningDirection(angle.Value), voteSourceID);
                        }
                    }
                    return action;
                }

                float calculateTurningDirection(float angle) {
                    if (angle < 0) {
                        return 1.0f;     //right
                    } else {
                        return -1.0f;    //left
                    }
                }
                
                public override void _ToDict(Dictionary<string, object> dict, bool saveLastAction){
                    dict["PriorityOnPotentionalWallCrash"] = priorityOnPotentionalWallCrash;
                    dict["CushionInSecondsToCrash"] = cushionInSecondsToCrash;
                }
                
                
                public override void _FromDict(Dictionary<string, object> dict){
                    string bName = this.GetType().Name;
                    priorityOnPotentionalWallCrash = Util.FloatFromJsonDict("PriorityOnPotentionalWallCrash", priorityOnPotentionalWallCrash, bName, dict);
                    cushionInSecondsToCrash = Util.FloatFromJsonDict("CushionInSecondsToCrash", cushionInSecondsToCrash, bName, dict);
                }
            }
        }
    }
}
