using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System;


namespace AT{
    namespace UBF{
        namespace Behaviors{

            /** Votes to do nothing with the desired priority.
             *
             * Not really useful by itself, but it is useful in combination with certain CompositeBehaviors/Arbiters. 
             * Is also useful for a Target while tuning other Agents.
             */
            public class DoNothing : AT.UBF.Behavior {

                public float priority = 0.1f;


                protected override AT.UBF.Action genActionInternal(AT.UBF.State state){
                    AT.UBF.Action action = new AT.UBF.Action();

                    action.SetTurnTracksRight(0.0f, voteSourceID);
                    action.SetTurnTurretRight(0.0f, voteSourceID);
                    action.SetDriveForward(0.0f, voteSourceID);
                    action.SetFireGun(false, voteSourceID);
                    action.SetPriority(priority, voteSourceID);

                    return action;
                }


                public override void _ToDict(Dictionary<string, object> dict, bool saveLastAction){
                    dict["Priority"] = priority;
                }

                /** Loads arguments from a dictionary, verifying as needed.
                 */
                public override void _FromDict(Dictionary<string, object> dict){
                    string bName = this.GetType().Name;
                    priority = Util.FloatFromJsonDict("Priority", priority, bName, dict);
                }
            }
        }
    }
}