using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System;


namespace AT{
    namespace UBF{
        namespace Behaviors{

            /** Drives in a straight line at the desired speed at all times.
             */
            public class DriveInAStraightLine : AT.UBF.Behavior {

                public float desiredSpeed = 1.0f;
                public float priority = 0.5f;

                protected override AT.UBF.Action genActionInternal(AT.UBF.State state){
                    AT.UBF.Action action = new AT.UBF.Action();

                    action.SetTurnTracksRight(0.0f, voteSourceID); // keep heading straight
                    action.SetDriveForward(desiredSpeed, voteSourceID);
                    action.SetPriority(priority, voteSourceID);

                    return action;
                }

                /** Save arguments to a dictionary.
                 */
                public override void _ToDict(Dictionary<string, object> dict, bool saveLastAction){
                    dict["DesiredSpeed"] = desiredSpeed;
                    dict["Priority"] = priority;
                }

                /** Loads arguments from a dictionary, verifying as needed.
                 */
                public override void _FromDict(Dictionary<string, object> dict){
                    string bName = this.GetType().Name;
                    priority = Util.FloatFromJsonDict("Priority", priority, bName, dict);
                    desiredSpeed = Util.FloatFromJsonDict("DesiredSpeed", desiredSpeed, bName, dict);
                }
            }
        }
    }
}