using UnityEngine;
using System;
using System.Collections;
using System.Collections.Generic;

namespace AT{
    namespace UBF{
        namespace Behaviors{
            /** If an enemy tank is in sight, maintain a desired distance from it. If needed drive directly toward/away
             * from it.
             */
            class MaintainDistanceToEnemy : AT.UBF.Behavior{

                public float desiredDistToEnemy = 4.0f;
                public float priorityOnEnemyFound = 0.9f;

                protected override AT.UBF.Action genActionInternal(AT.UBF.State state){
                    AT.UBF.Action action = new AT.UBF.Action();
                    if(state.visibleEnemyTanks.Count < 1){     
                        return action; // No enemy in sight; return NULL vote
                    }

                    Vector2 enemyPosition = state.visibleEnemyTanks[0].getPosition();

                    // Face the enemy tank as fast as possible
                    action.SetTurnTracksRight(
                        Util.findShortestTrackRotationToTarget(state.self.getPosition(), enemyPosition, state.self.getTracksRotation()),
                        voteSourceID);

                    // Maintain desired distance to enemy position (can be negative if we are too close)
                    float distToEnemy = (enemyPosition - state.self.getPosition()).magnitude;
                    float numTicksToDesiredPos = (distToEnemy - desiredDistToEnemy) / AT.Tank.maxMoveSpeed;
                    float driveForward = Mathf.Clamp(numTicksToDesiredPos, -1.0f, 1.0f); // clamp to max/min (valid) values
                    action.SetDriveForward(driveForward, voteSourceID);

                    action.SetPriority(priorityOnEnemyFound, voteSourceID);
                    return action;
                }

                public override void _ToDict(Dictionary<string, object> dict, bool saveLastAction){
                    dict["DesiredDistToEnemy"] = desiredDistToEnemy;
                    dict["PriorityOnEnemyFound"] = priorityOnEnemyFound;
                }

                public override void _FromDict(Dictionary<string, object> dict){
                    string bName = this.GetType().Name;
                    desiredDistToEnemy = Util.FloatFromJsonDict("DesiredDistToEnemy", desiredDistToEnemy, bName, dict);
                    priorityOnEnemyFound = Util.FloatFromJsonDict("PriorityOnEnemyFound", priorityOnEnemyFound, bName, dict);
                }
            }
        }
    }
}