﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

namespace AT{
    namespace UBF{
        namespace Behaviors{
            /** Controls player control of Tank (i.e. WASD and space bar/mouse button to fire the gun).
             *
             *  This is meant to be used on its own (single Behavior Agent).
             */
            public class PlayerControlled : AT.UBF.Behavior {
                
                float HandleTurretRotateInput(AT.UBF.State state){
                    // Compare angleToMouse to our turret rotation, return a turnRight vote that will make them match 
                    // (if difference is less than a single full speed roation (maxRot), return what fraction of full speed we should move (0.0 to 1.0)
                    Vector2 mousePos = Camera.main.ScreenToWorldPoint(Input.mousePosition);
                    float  turnRight = Util.findShortestTurretRotationToTarget(state.self.getPosition(), mousePos, state.self.getTurretRotation());
                    return turnRight;
                }

                float HandleDriveForwardInput(){
                    if (Input.GetKey (KeyCode.W)) { 
                        return 1.0f; // Forward
                    } else if (Input.GetKey(KeyCode.S)){ 
                        return -1.0f; // Reverse
                    }else{ 
                        return 0.0f; // Full Stop   
                    }
                }   

                float HandleTurnTracksInput(){
                    if (Input.GetKey (KeyCode.A)) { 
                        return -1.0f; // Turn Left
                    } else if (Input.GetKey(KeyCode.D)){ 
                        return 1.0f; // Turn Right
                    }
                    return 0.0f;
                }

                protected override AT.UBF.Action genActionInternal(AT.UBF.State state){
                    AT.UBF.Action action = new AT.UBF.Action();

                    action.turnTracksRight = HandleTurnTracksInput();
                    action.turnTurretRight = HandleTurretRotateInput(state);
                    action.driveForward    = HandleDriveForwardInput();

                    // Originally, this was on key/button UP but with rate limited firing, it felt weird. 
                    // We now provide a visual cue when a shot is ready to allow for more strategic shooting.
                    if (Input.GetMouseButton(0) || Input.GetKey (KeyCode.Space)) { // Left Mouse Button or Space Key
                        action.fireGun = true;
                    }

                    return action;
                }
            }
        }
    }
}
