using System;
using UnityEngine;
using UnityEditor;
using System.Collections;

namespace AT{
    namespace UBF{ 
        namespace Testing{
            public class IntegrationTest : MonoBehaviour {

                /** Perform any necessary initial setup. Call this yourself from RunTest if needed.
                 *
                 * For Integration tests, this typically requires loading an entire game state from file (and forcing the
                 * Agent(s) to generate an action based on that state). This default setup does just that given a path.
                 * RunTest just needs to find the Agent(s) it is interested in and grab its currAction.
                 */
                public virtual void SetupTest(string path){
                    AT.GameManager.instance.LoadGameFromDisc(path);
                    // Note: the game manager also forces the agent's state and currentAction members to update
                }

                /** Run the Unit test.
                 *
                 * Use Debug.Log and Debug.Assert as needed. 
                 */
                public virtual void RunTest(){
                    // This is provided as an example only
                    // This particular game state has the Agent in question having the enemy in their sights...
                    // so it should be trying to fire its' gun (ignoring that the gun is reloading in this case)
                    SetupTest("Moments/2015-12-04T00_26_00.json");
                    Debug.Log("Default Integration Test Running");

                    GameObject go = GameObject.Find("Agents-CircleAndShootSr.json"); // find the desired agent by name
                    Controller agent = go.GetComponent<Controller>() as Controller;
                    Debug.Assert(agent.currAction.fireGun.HasValue && (agent.currAction.fireGun.Value == true),
                        "CircleAndShoot did not try to Fire its' Gun while the enemy is in sight.");

                }
            }
        }
    }
}