﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System;
using System.Reflection;

namespace AT{
    namespace UBF{

        /** Action selection logic for CompositeBehaviors 
         *
         * There is no SaveToDict() method for Arbiters, because CompositeBehavior handles it. 
         */
        public class Arbiter {

            public int voteSourceID;

            /** Return an Action using the candidate Actions anyway you like.
             *
             *  You can select one Action among the candidates, combine them, or any other method 
             *  you can think of. However, you MUST return an Action.
             */
            public virtual AT.UBF.Action selectAction(List<AT.UBF.Action> candidateActions){
                return null; 
            }

            /** 
             */
            public Dictionary<string, object> ToDict(){
                Dictionary<string, object> dict = new Dictionary<string, object>();
                dict["ArbiterType"] = this.GetType().FullName; // Required 
                this._ToDict(dict);
                return dict;
            }

            public virtual void _ToDict(Dictionary<string, object> dict){
                // dict["VotePower"] = this.votePower; // Example
            }
            
            /** Creates a new Arbiter instance from the provided Dictionary.
             *
             * Any parsing errors, invalid or missing argument errors are caught here and NULL is 
             * returned. The error will be logged, but we do this because these errors are expected.
             */
            public static AT.UBF.Arbiter FromDict(Dictionary<string, object> dict){
                // Step 1: Find the fully specified Arbiter Class Name using the '_type' key
                string arbiterClassName;
                try{
                    arbiterClassName = dict["ArbiterType"] as string;
                }catch(KeyNotFoundException){
                    Debug.LogError("Did not find an 'ArbiterType' key specifying the Arbiter Class name.");
                    return null; 
                }

                // Step 2: Create a new instance of that Arbiter (cannot create instance of MonoBehavior derived classes)
                AT.UBF.Arbiter tempArbiter = (AT.UBF.Arbiter) Activator.CreateInstance(null, arbiterClassName).Unwrap();
                if(tempArbiter == null){ 
                    Debug.LogError("Could not find the Arbiter with Class name '" + arbiterClassName + "'");
                    return null; 
                }

                // Step 3: Setup vote source ID for identifying which Behavior/Arbiter a specific vote came from
                tempArbiter.voteSourceID = Controller.VoteSourceID;  // Controller auto-increments this ID

                // Step 4: Load any options, arguments and internal state to the Arbiter
                try{
                    tempArbiter._FromDict(dict);
                    return tempArbiter;
                }catch(Exception){
                    Debug.LogError("Failed to load Arbiter from Dictionary.");
                    return null;
                }
            }

            public virtual void _FromDict(Dictionary<string, object> dict){
                // this.votePower = (int) dict["VotePower"]; // Example
            }

            /** Return the _ToDict key:object values as a pretty string meant for the _ToDict Dialog Box.
             *
             *  Appends the info to infoStr and returns the new string.
             */
            public virtual string GetToDictDialogBoxString(string infoStr, bool shadowTwin){
                Dictionary<string, object> dict = new Dictionary<string, object>();
                _ToDict(dict);

                infoStr += String.Format("\n{0}{1}\n", this.GetType(), (shadowTwin ? "  (Shadow Twin)" : ""));
                foreach(KeyValuePair<string, object> entry in dict){
                    infoStr += String.Format("\t{0}: ", entry.Key);
                    infoStr += String.Format("{0}\n", entry.Value.ToString());
                }
                return infoStr;
            }
        }
    }
}
