using UnityEngine;
using UnityEditor;
using System;
using System.Collections;
using System.Collections.Generic;

namespace AT{
    namespace UBF{ 
        
        /** State related to a Tank (position, rotation, health, etc.).
         *
         * Previously, enemy tank state was available in a Triple<position, tracksRotation, velocity>.
         * This has been changed to an object to make changes to that info easier (hopefully).
         *
         * Also, attributes are now private with public getters to prevent accidental modification.
         */
        public class TankState{ 

            /** Private state attributes with their public getters 
             */
            private float turretRotation;
            public float getTurretRotation(){return turretRotation;}

            private float tracksRotation;
            public float getTracksRotation(){return tracksRotation;}

            private Vector2 position; // previously Triple.first
            public Vector2 getPosition(){return position;}

            private Vector2 velocity; // previously Triple.third
            public Vector2 getVelocity(){return velocity;}

            private int health; 
            public int getHealth(){return health;}

            private string color;
            public string getColor(){return color;}

            private AT.Team team;
            public AT.Team getTeam(){return team;}


            /** Constructor which will populate all required attributes.
             */
            public TankState(GameObject tankGameObj) {
                // Find the components we need to run the update first
                AT.Tank tankMB = tankGameObj.GetComponent<AT.Tank>();
                Rigidbody2D rigidbody = tankGameObj.GetComponent<Rigidbody2D>();
                AT.Health healthMB = tankMB.health;

                turretRotation = tankMB.turret_transform.rotation.eulerAngles.z;
                tracksRotation = rigidbody.transform.rotation.eulerAngles.z; 
                position = rigidbody.position;
                velocity = rigidbody.velocity;  
                health = healthMB.getHealth(); 
                color = tankMB.tankColor;
                team = tankMB.team;
                // getTankDimensions();    // TODO: Do we want this here?
            }

            /** Draw a Custom view of this State object in the Inspector.
             */
            public void DrawInspectorGUI(){  
                EditorGUILayout.LabelField( 
                    "Agent - Color: " + getColor() + " - Team: " + getTeam().ToString(), EditorStyles.boldLabel);

                GUILayout.BeginHorizontal();
                GUILayout.Space(30); // indent
                GUILayout.BeginVertical();

                EditorGUILayout.LabelField("Tracks Rotation", 
                    String.Format("{0,6:F2}", tracksRotation), 
                    EditorStyles.boldLabel);

                EditorGUILayout.LabelField("Turret Rotation", 
                    String.Format("{0,6:F2}", turretRotation), 
                    EditorStyles.boldLabel);

                EditorGUILayout.LabelField("Position", 
                    String.Format("X {0,6:F2}  Y {1,6:F2}", position.x, position.y), 
                    EditorStyles.boldLabel);

                EditorGUILayout.LabelField("Velocity", 
                    String.Format("X {0,6:F2}  Y {1,6:F2}", velocity.x, velocity.y), 
                    EditorStyles.boldLabel);

                EditorGUILayout.LabelField("Hit Points", 
                    String.Format("{0,6}", health), 
                    EditorStyles.boldLabel);

                GUILayout.EndVertical();
                GUILayout.EndHorizontal();
            }
        }
    }
}