using UnityEngine;
using UnityEditor;
using System.Collections;
using System.Collections.Generic;

namespace AT{
    public class UBFStateViewEditorWindow : EditorWindow{
        private Vector2 scrollPosition;
        

        [MenuItem ("Window/UBF State View ")]
        static void Init () {
            EditorWindow.GetWindow(typeof (UBFStateViewEditorWindow), false, "UBF State View");
        }

        /** Force the editor to redraw if the user selects a different GameObject in the Scene.
         */ 
        private void OnSelectionChange(){
            Repaint();
        }

        /** Force repaint on Update message IF the application is playing.
         */
        private void Update(){
            if(Application.isPlaying){
                Repaint();
            }
        }

        /** If the user has selected a GameObject that has a UBF Controller, display its' State (being passed to UBF tree).
         */
        private void OnGUI(){
            // Get the selected game object and try to get its' UBF Controller (stop rendering if it not found)
            GameObject obj = Selection.activeGameObject;
            if(obj == null){GUILayout.Label("Select a UBF Agent to see its' State."); return;}
            AT.UBF.Controller con = obj.GetComponent<AT.UBF.Controller>();
            if(con == null){GUILayout.Label("Select a UBF Agent to see its' State."); return;}
            if(!Application.isPlaying){GUILayout.Label("Hit play to see the UBF Agent's State."); return;}
            
            // This is likely to be big, so allow the user to scroll in both directions
            scrollPosition = EditorGUILayout.BeginScrollView(scrollPosition, true, true);

            con.state.DrawInspectorGUI();

            EditorGUILayout.EndScrollView();    
        }

    }
}