using UnityEngine;
using UnityEditor;
using System.Collections;
using System.Collections.Generic;

namespace AT{
    public class UBFTreeViewEditorWindow : EditorWindow{
        private Vector2 scrollPosition;
        

        [MenuItem ("Window/UBF Tree View ")]
        static void Init () {
            EditorWindow.GetWindow(typeof (UBFTreeViewEditorWindow), false, "UBF Tree View");
        }

        /** Force the editor to redraw if the user selects a different GameObject in the Scene.
         */ 
        private void OnSelectionChange(){
            Repaint();
        }

        /** Force repaint on Update message IF the application is playing.
         */
        private void Update(){
            if(Application.isPlaying){
                Repaint();
            }
        }

        /** If the user has selected a GameObject that has a UBF Controller, display its' tree.
         */
        private void OnGUI(){
            // Get the selected game object and try to get its' UBF Controller (stop rendering if it not found)
            GameObject obj = Selection.activeGameObject;
            if(obj == null){GUILayout.Label("Select a UBF Agent to see its' tree."); return;}
            AT.UBF.Controller con = obj.GetComponent<AT.UBF.Controller>();
            if(con == null){GUILayout.Label("Select a UBF Agent to see its' tree."); return;}
            if(!Application.isPlaying){GUILayout.Label("Hit play to see the UBF Agent's tree."); return;}

            // Because of how Unity forces us to draw this, we need to know big this tree diagram will be.
            int viewWidth  = con.behavior.GetNumLeafBehaviors() * con.behavior.GetUiWPlusMargin();
            int viewHeight = con.behavior.GetHeight() * con.behavior.GetUiHPlusMargin();
            
            // This is likely to be big, so allow the user to scroll in both directions
            scrollPosition = EditorGUILayout.BeginScrollView(scrollPosition, true, true);

            BeginWindows();
            Rect currRect = new Rect(0, 10, viewWidth, viewHeight+10); // Add a 10px margin at the top 
            GUI.Box(currRect, ""); // show the calculated view Width and Height via GUI.Box surrounding all of the tree
            con.behavior.DrawAdvInspectorGUI(currRect);
            EndWindows();

            // Force the scroll bars to extend to cover ALL of our content (ugly hack, but I can't find another solution)
            GUILayout.Label("", GUILayout.Width(viewWidth), GUILayout.Height(viewHeight));

            EditorGUILayout.EndScrollView();    
        }

    }
}