﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

namespace AT{

    /** Handles initial setup of Bullet and Collisions with Walls (but not Tanks).
     *
     *  Walls always destroy the Bullet. 
     *  Tanks handle bullet collisions themselves (see Health script).
     */
    public class Bullet : MonoBehaviour {

        private bool loadedFromDict = false;

        /** Set bullet's time to live and set initial force for the physics system.
         */
        void Start () {
            Destroy (gameObject, GameManager.instance.bulletSecondsToLive);
            if(!loadedFromDict){ // Only add initial force if this wasn't loaded from a Save file
                GetComponent<Rigidbody2D>().AddForce(new Vector2(transform.right.x * GameManager.instance.bulletSpeed, transform.right.y * GameManager.instance.bulletSpeed));  
            }  
        }

        /** Bullets are Triggers, so we need to use this to check for "collisions." 
         *
         *  Walls always destroy bullets.
         */
        void OnTriggerEnter2D(Collider2D c){
            if (c.gameObject.tag == MapManager.instance.WallTag) {
                Destroy (gameObject);
            }
        }

        public Dictionary<string, object> ToDict(){
            Dictionary<string, object> dict = new Dictionary<string, object>();

            dict["_Type"] = this.GetType().FullName;

            dict["posX"] = transform.position.x;
            dict["posY"] = transform.position.y;
            dict["posZ"] = transform.position.z;

            Rigidbody2D rb = GetComponent<Rigidbody2D>();
            dict["velocityX"] = rb.velocity.x;
            dict["velocityY"] = rb.velocity.y;

            return dict;
        }

        public void FromDict(Dictionary<string, object> dict){
            loadedFromDict = true;
            transform.position = new Vector3((float) ((double)dict["posX"]), (float) ((double)dict["posY"]), (float) ((double)dict["posZ"]));
            Rigidbody2D rb = GetComponent<Rigidbody2D>();
            rb.velocity = new Vector2((float) ((double)dict["velocityX"]), (float) ((double)dict["velocityY"]));
            // TODO: Time to live?
        }
    }
}
