﻿using UnityEngine;
using System.Collections;

namespace AT{
    /** Handles HP, Damage and destroys the GameObect if too much is taken.
     * 
     * Sends health updates via BroadcastMessage("UpdateHealth", health) to those who are interested.
     * As of this writing, this is only used by HealthBar.
     */
    public class Health : MonoBehaviour {
        private int originalHealth;
        private int health;
        private int bulletDamage;
        private int tankDamage;

        public void config(int health, int originalHealth, int bulletDamage, int tankDamage){
            this.health = health;
            this.originalHealth = originalHealth;
            this.bulletDamage = bulletDamage;
            this.tankDamage = tankDamage;
            // initial update since this agent might have been loaded from file
            BroadcastMessage("UpdateHealth", health/((float) originalHealth)); 
        }

        public int getHealth(){
            return this.health;
        }

        /** Handles Tank-on-Tank collision damage.
         */
        void OnCollisionEnter2D(Collision2D c){
            if (c.gameObject.tag == "Tank") {
                TakeDamage(tankDamage);
            }else if(c.gameObject.tag == "Wall"){
                ;// Do nothing; Walls do not cause damage
            }
            //Debug.Log("Collision Entered with a " + c.gameObject.tag);
        }

        /**
         *
         * Note: This is called each frame where a collider or another object is touching this object's collider. 
         * OnCollisionEnter2D is called only once per collision!
         */
        void OnCollisionStay2D(Collision2D c){
            //Debug.Log("Collision continuing with a " + c.gameObject.tag);
        }

        /** Handles Bullet-on-Tank collision damage.
         */
        void OnTriggerEnter2D(Collider2D c){
            if (c.gameObject.tag == "Bullet") {
                Destroy (c.gameObject);
                TakeDamage(bulletDamage);
            }
        }

        /** Subtract damage from HP, send the updated HP to listeners and destroy this object if dead.
         */
        private void TakeDamage(int damage){
            health -= damage;
            // notify listeners of new heath value as percentage; force floating point division
            BroadcastMessage("UpdateHealth", health/((float) originalHealth)); 
            if (health <= 0) {
                Destroy (gameObject);
            }
        }
    }
}
