using UnityEngine;
using System.Collections;
using UnityEditor;

namespace AT {
    /** Provides improved Inspector for MapManager including Load/Clear and current Map path.
     *
     */
    [CustomEditor(typeof(MapManager))]
    public class MapManagerEditor : Editor {

        public override void OnInspectorGUI(){
            MapManager manager = (MapManager) target;
            DrawDefaultInspector();

            GUI.enabled = false;
            GUILayout.TextArea("This assumes (and enforces) all map paths are relative and are in the Maps directory.");
            GUI.enabled = true;

            GUILayout.BeginHorizontal();
            GUILayout.Label("Map", EditorStyles.boldLabel);

            // Don't allow map load/clear during play (though MapManager CAN do this, it should only be done when 
            // loading from a file, etc.)
            if(Application.isPlaying){ GUI.enabled = false; } 
            if(GUILayout.Button("Load")){
                   LoadMapDialog();
            }
            if(GUILayout.Button("Clear")){
                ClearMapDialog();
            }
            if(GUILayout.Button("Reload")){
                manager.ReloadMap();
            }
            GUI.enabled = true;

            GUILayout.EndHorizontal();
            GUILayout.BeginHorizontal();
            GUILayout.Label("Current Map:", EditorStyles.boldLabel);
            GUILayout.Label(manager.CurrMapFilePath);
            GUILayout.EndHorizontal();
        }

        /** Load a map to the current Scene from local flat file.
         */
        private void LoadMapDialog(){
            MapManager manager = (MapManager) target;
            string path = EditorUtility.OpenFilePanel(
                            "Choose Map File",
                            "Maps/",
                            "txt");
            if (path.Length != 0) {
                    manager.CurrMapFilePath = path;
                    EditorUtility.SetDirty(target); // make sure changed attributes are preserved on the object
            }
        }

        /** Delete the current map from the current Scene.
         */
        private void ClearMapDialog(){
            MapManager manager = (MapManager) target;
            manager.ClearMap();
        }

    }
}