

namespace AT{

    /** Represents a discrete message sent over an unspecified communication channel from one tank to another.
     *
     * - This is meant to be read-only after creation (i.e. no setters beyond the constructor).
     * - The sourceID is provided because this will identify the exact Behavior that sent the message, whereas the 
     *   sender will only identify the team, or tank (e.g. "blue").
     */
    public class Message{
        public enum ReceiverType{Team, Individual};

        // ID of the exact Behavior that sent this message
        private int sourceID;
        public int getSourceID(){return sourceID;}

        // ID/color of the tank that sent this message
        private string sender;
        public string getSender(){return sender;}

        // Intended for either the whole team or a specific tank (color/ID)
        private AT.Message.ReceiverType receiverType;
        public AT.Message.ReceiverType getReceiverType(){return receiverType;}

        // Intended receiver's ID (if a team, this is a string representation of the enum, if an individual it's a color/ID) 
        private string receiver;
        public string getReceiver(){return receiver;}

        // Message must be a string; however you can easily encode more complex data strcutres via CSV, JSON, etc.
        private string message;
        public string getMessage(){return message;}

        public Message(int sourceID, string sender, AT.Message.ReceiverType receiverType, string receiver,  string message){
            this.sourceID = sourceID;
            this.sender = sender;
            this.receiverType = receiverType;
            this.receiver = receiver;
            this.message = message;
        }
    }
}